package net.wanji.common.enums;

import java.util.Objects;

/**
 * 灯组位置枚举
 *
 * @author Kent HAN
 * @date 2023/1/5 8:54
 */
public enum LightsAddressEnum {
    ONE(1, "1", "北"),
    TWO(2, "2", "东北"),
    THREE(3, "3", "东"),
    FOUR(4, "4", "东南"),
    FIVE(5, "5", "南"),
    SIX(6, "6", "西南"),
    SEVEN(7, "7", "西"),
    EIGHT(8, "8", "西北"),
    ONE_ONE(11, "11", "北一次过街"),
    ONE_TWO_ONE(121, "121", "北出口二次过街"),
    ONE_TWO_TWO(122, "122", "北进口二次过街"),
    TWO_ONE(21, "21", "东北一次过街"),
    TWO_TWO_ONE(221, "221", "东北出口二次过街"),
    TWO_TWO_TWO(222, "222", "东北进口二次过街"),
    THREE_ONE(31, "31", "东一次过街"),
    THREE_TWO_ONE(321, "321", "东出口二次过街"),
    THREE_TWO_TWO(322, "322", "东进口二次过街"),
    FOUR_ONE(41, "41", "东南一次过街"),
    FOUR_TWO_ONE(421, "421", "东南出口二次过街"),
    FOUR_TWO_TWO(422, "422", "东南进口二次过街"),
    FIVE_ONE(51, "51", "南一次过街"),
    FIVE_TWO_ONE(521, "521", "南出口二次过街"),
    FIVE_TWO_TWO(522, "522", "南进口二次过街"),
    SIX_ONE(61, "61", "西南一次过街"),
    SIX_TWO_ONE(621, "621", "西南出口二次过街"),
    SIX_TWO_TWO(622, "622", "西南进口二次过街"),
    SEVEN_ONE(71, "71", "西一次过街"),
    SEVEN_TWO_ONE(721, "721", "西出口二次过街"),
    SEVEN_TWO_TWO(722, "722", "西进口二次过街"),
    EIGHT_ONE(81, "81", "西北一次过街"),
    EIGHT_TWO_ONE(821, "821", "西北出口二次过街"),
    EIGHT_TWO_TWO(822, "822", "西北进口二次过街");

    private final int code;
    private final String msg;
    private final String description;

    LightsAddressEnum(int code, String msg, String description) {
        this.code = code;
        this.msg = msg;
        this.description = description;
    }

    public static String getMsgByCode(int code) {
        for (LightsAddressEnum value : LightsAddressEnum.values()) {
            if (value.code == code) {
                return value.msg;
            }
        }
        return null;
    }

    public static Integer getCodeByMsg(String msg) {
        for (LightsAddressEnum value : LightsAddressEnum.values()) {
            if (Objects.equals(value.msg, msg)) {
                return value.code;
            }
        }
        return null;
    }

    public String getMsg() {
        return msg;
    }
}
