package net.wanji.common.gts;

import org.locationtech.jts.geom.Geometry;
import org.locationtech.jts.io.ParseException;
import org.locationtech.jts.io.WKTReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class GeometryUtil {

	public final static double PI = 3.1415926535898D;
	private static Logger log = LoggerFactory.getLogger(GeometryUtil.class);

	public static Geometry str2Geometry(String coors) {
		Geometry geometry = null;
		int num = getSubNumber(coors, ";");
		try {
			if (num == 0) {// 点
				geometry = getTheGeom(coors, 1);
			} else if (num > 0) {
				String[] _s = coors.split(";");
				if (_s[0].equals(_s[_s.length - 1])) {// 面
					geometry = getTheGeom(coors, 3);
				} else {// 线
					geometry = getTheGeom(coors, 2);
				}
			}
		} catch (Exception e) {
			log.error("str2Geometry", e);
		}
		return geometry;
	}

	public static int getSubNumber(String des, String reg) {
		Pattern p = Pattern.compile(reg);
		Matcher m = p.matcher(des);
		int count = 0;
		while (m.find()) {
			count++;
		}
		return count;
	}

	public static Geometry getTheGeom(String coors, int type) throws ParseException {
		coors = coors.replaceAll(";", ",");
		String[] xys = coors.split(",");
		StringBuffer xy = null;
		String endString = "";
		if (type == 1) {// 点
			xy = new StringBuffer("POINT(");
			endString = ")";
		} else if (type == 2) {// 线
			xy = new StringBuffer("LINESTRING(");
			endString = ")";
		} else if (type == 3) {// 面
			xy = new StringBuffer("POLYGON((");
			String x = xys[0];
			String y = xys[1];
			String endy = xys[xys.length - 1];
			if (y.equals(endy)) {
				endString = "))";
			} else {
				endString = "," + x + " " + y + "))";
			}
		}
		for (int i = 0; i < xys.length; i += 2) {
			String x = xys[i];
			String y = xys[i + 1];
			xy.append(x);
			xy.append(" ");
			xy.append(y);
			xy.append(",");
		}
		String wkt = xy.substring(0, xy.length() - 1) + endString;
		Geometry geometry = new WKTReader().read(wkt);
		geometry.setSRID(4326);
		return geometry;
	}

	public static double getDistance(double x1, double y1, double x2, double y2) {
		double latRadians1 = y1 * (Math.PI / 180);
		double latRadians2 = y2 * (Math.PI / 180);
		double latRadians = latRadians1 - latRadians2;
		double lngRadians = x1 * (Math.PI / 180) - x2 * (Math.PI / 180);
		double f = 2 * Math.asin(Math.sqrt(Math.pow(Math.sin(latRadians / 2), 2) + Math.cos(latRadians1) * Math.cos(latRadians2) * Math.pow(Math.sin(lngRadians / 2), 2)));
		return f * 6378137;
	}

	public static double calcAngle(double fStartLon, double fStartLat, double fEndLon, double fEndLat) {
		double fAngle = 0;
		if (fEndLon != fStartLon) {
			double fAtan = (fEndLat - fStartLat) / (fEndLon - fStartLon);
			fAngle = Math.atan(fAtan);
			if (fEndLon - fStartLon < 0)
				fAngle += PI;
			else if (fAngle < 0)
				fAngle += 2 * PI;
		} else {
			if (fEndLat > fStartLat) {
				fAngle = PI / 2;
			} else if (fEndLat < fStartLat) {
				fAngle = PI / 2 * 3;
			} else if (fEndLat == fStartLat) {
				fAngle = 0;
			}
		}
		return fAngle;
	}
}
